@@ -17,17 +17,20 @@ import os
 import sys
 import logging
 import subprocess
+import cysecuretools.imgtool_1_4_0.main as imgtool
+import cysecuretools.execute.encrypted_image_runner as encryptor
 from shutil import copy2
 from pathlib import Path
 from intelhex import hex2bin, bin2hex
-import cysecuretools.execute.encrypted_image_runner as encryptor
 from cysecuretools.targets.common.policy_parser import PolicyParser, ImageType, KeyType
+from cysecuretools.targets.cy8ckit_064x0s2_4343w.target_builder import CY8CKIT_064X0S2_4343W_Builder
+from cysecuretools.targets.cyb06xx5.target_builder import CYB06xx5_Builder
 
 logger = logging.getLogger(__name__)
 
 
 class SignTool:
-    def __init__(self, policy_file, memory_map):
+    def __init__(self, policy_file, memory_map, target_builder):
         # Resolve paths
         self.policy_dir = os.path.dirname(Path(policy_file).absolute())
         self.PKG_PATH = os.path.dirname(os.path.realpath(__file__))
@@ -36,6 +39,12 @@ class SignTool:
         self.policy = PolicyParser.get_json(policy_file)
         self.memory_map = memory_map
 
+        # Temporary solution to use different version of imgtool for specific device
+        self.use_default_imgtool = type(target_builder) is not CY8CKIT_064X0S2_4343W_Builder and \
+                                   type(target_builder) is not CYB06xx5_Builder
+        if not self.use_default_imgtool:
+            self.IMG_TOOL_PATH = os.path.join(self.PKG_PATH, '../imgtool_1_4_0/main.py')
+
     def sign_image(self, hex_file, image_id):
         """
         Signs hex file with the key specified in the policy file.
@@ -76,11 +85,9 @@ class SignTool:
                     hex2bin(hex_out, bin_out)
                     bin2hex(bin_out, output_name, offset=int(image['address']))
                     os.remove(bin_out)
-                    logger.info(f'Image UPGRADE: {hex_out}\n')
 
                 # Replace input hex file with the
             result.append(hex_out)
-        os.remove(unsigned_boot_hex)
         result = tuple(result) if len(result) > 0 else None
         return result
 
@@ -104,35 +111,60 @@ class SignTool:
         if hex_out is None:
             hex_out = '{0}_{2}{1}'.format(*os.path.splitext(hex_in) + ('signed',))
 
-        args = [
-             sys.executable, self.IMG_TOOL_PATH,
-             'sign',
-             '--key', key.pem_key_path,
-             '--header-size', hex(self.memory_map.MCUBOOT_HEADER_SIZE),
-             '--pad-header',
-             '--align', '8',
-             '--version', slot['version'],
-             '--image-id', str(slot['id']),
-             '--rollback_counter', str(slot['rollback_counter']),
-             '--slot-size', hex(size),
-             '--overwrite-only',
-             hex_in,
-             hex_out
-        ]
+        if self.use_default_imgtool:
+            args = [
+                 sys.executable, self.IMG_TOOL_PATH,
+                 'sign',
+                 '--key', key.pem_key_path,
+                 '--header-size', hex(self.memory_map.MCUBOOT_HEADER_SIZE),
+                 '--pad-header',
+                 '--align', '8',
+                 '--version', slot['version'],
+                 '--image-id', str(slot['id']),
+                 '--rollback_counter', str(slot['rollback_counter']),
+                 '--slot-size', hex(size),
+                 '--overwrite-only',
+                 hex_in,
+                 hex_out
+            ]
+        else:
+            args = [
+                '--key', key.pem_key_path,
+                '--header-size', hex(self.memory_map.MCUBOOT_HEADER_SIZE),
+                '--pad-header',
+                '--align', '8',
+                '--version', slot['version'],
+                '--slot-size', hex(size),
+                '--overwrite-only',
+                '--erased-val', '0',
+                hex_in,
+                hex_out,
+
+                # Add Cypress TLV
+                '--imageid', 'B', str(slot['id']),
+                '--rollbackcounter', 'B', str(slot['rollback_counter']),
+            ]
+
         if image_type != ImageType.BOOT.name:
             args.append('--pad')
         logger.debug(f'Run imgtool with arguments: {args}')
 
-        process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-        stderr = process.communicate()[1]  # catch stderr outputs
-        rc = process.wait()
+        if self.use_default_imgtool:
+            process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            stderr = process.communicate()[1]  # catch stderr outputs
+            rc = process.wait()
+        else:
+            try:
+                imgtool.sign(args)
+            except SystemExit as e:
+                rc = e.code
 
         if rc != 0:
             logger.error('Signature is not added!')
             logger.error(f'Message from imgtool: {stderr.decode("utf-8")}')
             logger.error('imgtool finished execution with errors!')
         else:
-            logger.info(f'Image for slot {image_type} signed successfully!')
+            logger.info(f'Image for slot {image_type} signed successfully! ({hex_out})')
             return hex_out
 
     def encrypt_image(self, slot, image_type, hex_in):
@@ -164,7 +196,8 @@ class SignTool:
             '--rlb-count', str(slot['rollback_counter']),
             '--slot-size', hex(size),
             '--img-offset', address,
-            '--pad', 1
+            '--pad', 1,
+            '--imgtool-path', self.IMG_TOOL_PATH
         ]
 
         logger.debug(f'Run encryption with arguments: {args}')
